---------------------------------------------------------------
-- NON RESTORING Divisor for normalized (and positive) operands
-- operand X is 0,01xxxxxx and Y = 0,1xxxxxxx
-- 
-- Return Quotient Q of PBITS and remainder R of NBITS
---------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;
use work.mypackage.all;

entity div_nr_f_pipe is
    port (
        A: in STD_LOGIC_VECTOR (NBITS-1 downto 0);
        B: in STD_LOGIC_VECTOR (NBITS-1 downto 0);
        clk: in STD_LOGIC;
        Q: out STD_LOGIC_VECTOR (PBITS-1 downto 0);
        R: out STD_LOGIC_VECTOR (NBITS-1 downto 0)
     );
end div_nr_f_pipe;

architecture mult_arch of div_nr_f_pipe is

component sum
   PORT(
      op_a : IN std_logic_vector(NBITS-1 downto 0);
      op_b : IN std_logic_vector(NBITS downto 0);
      x_i : IN std_logic;          
      sal : OUT std_logic_vector(NBITS downto 0)
      );
end component;

component s_r
    port (
        op_a: in STD_LOGIC_VECTOR (NBITS downto 0);
        op_m: in STD_LOGIC_VECTOR (NBITS downto 0);
        sr: in STD_LOGIC;
        sal: out STD_LOGIC_VECTOR (NBITS downto 0)
     );
end component;

component registers is
    generic (LENGHT : integer );
    Port ( clk : in std_logic;
           entrada : in std_logic_vector(LENGHT-1 downto 0);
           salida : out std_logic_vector(LENGHT-1 downto 0)
           );
end component;

component registers_m is
    generic (LENGHT : integer );
    Port ( clk : in std_logic;
           entrada : in std_logic_vector(LENGHT-2 downto 0);
           ultima_entrada : in std_logic;
           salida : out std_logic_vector(LENGHT-1 downto 0)
           );
end component;

component register_1 is
    Port ( clk : in std_logic;
           entrada : in std_logic;
           salida : out std_logic
           );
end component;


type matrizconexion is array (0 to PBITS) of STD_LOGIC_VECTOR (NBITS downto 0);
type matrizconexion_P is array (0 to PBITS) of STD_LOGIC_VECTOR (PBITS downto 0);

Signal YY_in: matrizconexion;
Signal QQ_in: matrizconexion_P;
Signal m_cablesIn, m_cablesOut: matrizconexion;

Signal  S_o_R: STD_LOGIC_VECTOR (PBITS downto 0);
Signal  final_outp: STD_LOGIC_VECTOR (NBITS downto 0);
signal  adjust: STD_LOGIC;

Signal  QQ: STD_LOGIC_VECTOR (PBITS-1 downto 0);
Signal  ceros: STD_LOGIC_VECTOR (NBITS-1 downto 0);
Signal  RR_I: STD_LOGIC_VECTOR (NBITS-1 downto 0);
Signal  YY,XX: STD_LOGIC_VECTOR (NBITS-1 downto 0);

begin

   process (CLK)
   begin
      if CLK'event and CLK='0' then  --CLK rising edge
         XX <= A;  YY <= B;
         Q <= QQ; R <= RR_I;
      end if;
   end process;

   S_o_R(0) <= '0'; 
   ceros <= (others => '0'); 
   m_cablesIn(0) <= '0' & XX;
   YY_IN(0) <= '0' & YY;

   divider: for I in 0 to PBITS-1 generate
      int_mod: S_R port map (op_a => m_cablesIn(I),
          op_m => YY_IN(i), 
          sr => S_o_R(I),
          sal => m_cablesOut(I) );
   end generate;

   adjust <= m_cablesIn(PBITS)(NBITS);
   
   final_adjust: SUM port map (op_a => m_cablesIn(PBITS)(NBITS-1 downto 0),
       op_b => YY_IN(PBITS), 
       x_i => adjust,
       sal => final_outp );
   
   --signal conections
   conex: for I in 0 to PBITS-2 generate
        if_gen: if (I+1) mod DEPTH = 0 generate
            reg1: registers generic map (LENGHT=>NBITS) 
                               port map (clk => clk, entrada => m_cablesOut(i)(NBITS-1 downto 0),
                                         salida => m_cablesIn(i+1)(NBITS downto 1) );
                  m_cablesIn(i+1)(0) <= '0';
            reg2: registers generic map (LENGHT=>NBITS+1) 
                               port map (clk => clk, entrada => YY_IN(i), 
                                         salida => YY_IN(i+1) );

            QQ_in(i)(I) <= m_cablesOut(i)(NBITS);

            reg3: registers generic map (LENGHT=>I+1) 
                               port map (clk => clk, entrada => QQ_in(i)(I downto 0) , 
                                         salida =>QQ_in(i+1)(I downto 0) );
            S_o_R(i+1) <= QQ_in(i+1)(I); 
        end generate;
        else_gen: if ((I+1) mod DEPTH) /= 0 generate
            S_o_R(i+1) <= m_cablesOut(i)(NBITS); 
            m_cablesIn(i+1) <= m_cablesOut(i)(NBITS-1 downto 0) & '0';
            YY_IN(i+1) <= YY_IN(i);

            prim: if I = 0 generate
                     QQ_in(i+1)(0) <= S_o_R(i+1);
            end generate;
            rest: if I > 0 generate
                     QQ_in(i+1)(I downto 0) <= S_o_R(i+1) & QQ_in(i)(I-1 downto 0);
            end generate;

        end generate;
     end generate;
   
   last_conex_if_gen: if DEPTH = 1 generate
            reg5 : registers generic map (LENGHT=>NBITS+1) port map ( clk => clk, entrada => m_cablesOut(PBITS-1), salida => m_cablesIn(PBITS) );
            reg2b: registers generic map (LENGHT=>NBITS+1) port map ( clk => clk, entrada => YY_IN(PBITS-1), salida => YY_IN(PBITS) );
            QQ_in(PBITS-1)(PBITS-1) <= m_cablesOut(PBITS-1)(NBITS);
            reg3b: registers generic map (LENGHT=>PBITS) port map ( clk => clk, entrada => QQ_in(PBITS-1)(PBITS-1 downto 0) , 
                                          salida => QQ_in(PBITS)(PBITS-1 downto 0) );
   end generate;
   last_conex_else_gen: if DEPTH /= 1 generate
            m_cablesIn(PBITS) <= m_cablesOut(PBITS-1);
            YY_IN(PBITS) <= YY_IN(PBITS-1); 
            QQ_in(PBITS)(PBITS-1 downto 0) <= m_cablesOut(PBITS-1)(NBITS) & QQ_in(PBITS-1)(PBITS-2 downto 0);
   end generate;

   RR_I <= final_outp(NBITS-1 downto 0);

   quotient: for I in 0 to PBITS-1 generate
      QQ(I) <= not QQ_in(PBITS)(PBITS-1-I);
   end generate;

end mult_arch;


