
-- VHDL Test Bench for pipelined Divider
--
-- Notes: 
-- Testbench works for NBITS < 32	
--
LIBRARY  IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_arith.all;
USE IEEE.std_logic_unsigned.all;
USE work.mypackage.all;

LIBRARY ieee;
USE IEEE.STD_LOGIC_TEXTIO.ALL;
USE STD.TEXTIO.ALL;

ENTITY testb_tb IS
END testb_tb;

ARCHITECTURE pruebas OF testb_tb IS 
FILE RESULTS: TEXT OPEN WRITE_MODE IS "results.txt";

	COMPONENT div_nr_f_pipe
	PORT(
		a : IN std_logic_vector(NBITS-1 downto 0);
		b : IN std_logic_vector(NBITS-1 downto 0);
		clk : IN std_logic;          
		q : OUT std_logic_vector(PBITS-1 downto 0);
		r : OUT std_logic_vector(NBITS-1 downto 0)
		);
	END COMPONENT;

	SIGNAL x :  std_logic_vector(NBITS-1 downto 0);
	SIGNAL y :  std_logic_vector(NBITS-1 downto 0);
	SIGNAL clk :  std_logic;
	SIGNAL q :  std_logic_vector(PBITS-1 downto 0);
	SIGNAL r :  std_logic_vector(NBITS-1 downto 0);

BEGIN

	uut: div_nr_f_pipe PORT MAP(
		a => x,
		b => y,
		clk => clk,
		q => q,
		r => r
	);



	PROCESS -- clock process for clk,
	BEGIN
		CLOCK_LOOP : LOOP
		clk <= transport '0';
		WAIT FOR 10 ns;
		clk <= transport '1';
		WAIT FOR 10 ns;
		WAIT FOR 40 ns;
		clk <= transport '0';
		WAIT FOR 40 ns;
		END LOOP CLOCK_LOOP;
	END PROCESS;



-- *** Test Bench - User Defined Section ***
   tb_gen : PROCESS
	VARIABLE TX_LOC : LINE;
	VARIABLE TX_STR : String(1 to 4096);
   BEGIN
		for I in 2**(NBITS-1)  to 2**NBITS -1 loop
			for J in 2**(NBITS-1) to 2**NBITS -1 loop
				x <= CONV_STD_LOGIC_VECTOR (I, NBITS);
				y <= CONV_STD_LOGIC_VECTOR (J, NBITS);
				WAIT FOR 100 ns;
     		end loop;
   	end loop;

		WAIT FOR 200 ns;
      --wait; -- will wait forever
   END PROCESS;

   tb_test : PROCESS
	VARIABLE TX_LOC : LINE;
	VARIABLE TX_STR : String(1 to 4096);
   BEGIN
		WAIT FOR PBITS/DEPTH*100 ns;
		if DEPTH = 1 then 
			WAIT FOR 100 ns;
		end if;
		for I in 2**(NBITS-1)  to 2**NBITS -1 loop
			for J in 2**(NBITS-1) to 2**NBITS -1 loop
				WAIT FOR 100 ns;

				IF ((I*(2**(PBITS-1))) /= (J * CONV_INTEGER(Q)) + CONV_INTEGER(R)) THEN 
					write(TX_LOC,string'("ERROR!!! X=")); write(TX_LOC, X);
					write(TX_LOC,string'(" Y=")); write(TX_LOC, Y);
					write(TX_LOC,string'(" Q=")); write(TX_LOC, Q);
					write(TX_LOC,string'(" R=")); write(TX_LOC, R);
					write(TX_LOC, string'(" "));
					TX_STR(TX_LOC.all'range) := TX_LOC.all;
					writeline(results, TX_LOC);
					Deallocate(TX_LOC);
					ASSERT (FALSE) REPORT TX_STR SEVERITY FAILURE;
			   ELSIF (J < CONV_INTEGER(R)) THEN 
						write(TX_LOC,string'("--> Error Resto Mayor que Y =")); write(TX_LOC, 0.0);
						write(TX_LOC,string'("ns X=")); write(TX_LOC, X);
						write(TX_LOC,string'(" Y=")); write(TX_LOC, Y);
						write(TX_LOC,string'(" Q=")); write(TX_LOC, Q);
						write(TX_LOC,string'(" R=")); write(TX_LOC, R);
						write(TX_LOC, string'(" "));
						TX_STR(TX_LOC.all'range) := TX_LOC.all;		
						writeline(results, TX_LOC);
						Deallocate(TX_LOC);
						ASSERT (FALSE) REPORT TX_STR SEVERITY ERROR;
--					ELSE
--						write(TX_LOC,string'("OK -> X=")); write(TX_LOC, X);
--						write(TX_LOC,string'(" Y=")); write(TX_LOC, Y);
--						write(TX_LOC,string'(" Q=")); write(TX_LOC, Q);
--						write(TX_LOC,string'(" R=")); write(TX_LOC, R);
--						write(TX_LOC, string'(" "));
--						TX_STR(TX_LOC.all'range) := TX_LOC.all;
--						writeline(results, TX_LOC);
--						Deallocate(TX_LOC);
--						ASSERT (FALSE) REPORT TX_STR SEVERITY WARNING;

				END IF;

    		end loop;
   	end loop;
		ASSERT (FALSE) REPORT
		"Simulation successful (not a failure).  No problems detected. "
		SEVERITY FAILURE;
      --wait; -- will wait forever
   END PROCESS;




END;
